
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "process.h"

#include <common/build.h>
#include <common/exception.h>

using namespace bdk;

int Process::exec(std::vector < std::string > args)
{
	char* cargs[0xff];

	BC_ASSERT((sizeof(cargs) * sizeof(*cargs)) > args.size());
		
	pid_t pid = fork();
	if(pid < 0)
		throw BCSystemException("Process fork failed");

	
	if( 0 == pid ) 
	{
		// child	
		const char* env[] = {"PATH=/sbin:/usr/sbin:/bin:/usr/bin", NULL};
		cargs[0] = (char*)m_name.c_str();
		char** pargs = &cargs[1];

		std::vector < std::string >::iterator i = args.begin();
		std::vector < std::string >::iterator end = args.end();

		while(i != end)
		{
			*pargs++ = (char*)i->c_str();
			++i;
		}

		*pargs = NULL;

#ifdef BC_DEBUG
		BC_DEBUG_PRINT(("Debug: exec process %s with args: ", m_name.c_str()));
		char** parg = &cargs[0];
		while(*parg != NULL)
		{
			BC_DEBUG_PRINT(("\"%s\" ", *parg));
			++parg;
		}
		BC_DEBUG_PRINT(("NULL\n"));
#endif

		execve(m_name.c_str(), (char* const*)cargs, (char* const*)env);
		
		// gone into exec
		// of failed, this will be reached.
		throw BCSystemException("execve failed");
	}

	// parent
	// catch child exit status
	int status;
	waitpid( pid, &status, 0);
	return WEXITSTATUS(status);
}
